#include <ndis.h>
#include "ndisioct.h"

#define MAX_ENET_FRAMESIZE          ((USHORT)1514)
#define MIN_ENET_FRAMESIZE          14
#define MIN_LEGAL_ENET_FRAMESIZE    60
#define LOOK_AHEAD_BUFFER_SIZE  2048
#define MAX_ENET_FRAMEDATA_SIZE     1500
#define MAX_MULTICAST_ADDRESS   16

static const NDIS_PHYSICAL_ADDRESS  MaxPhys = 
   NDIS_PHYSICAL_ADDRESS_CONST(-1,-1);
static const NDIS_STRING VendorName = 
   NDIS_STRING_CONST("NDISIOCTL Test Miniport");
static const NDIS_OID SupportedOids[] = 
{
    OID_GEN_SUPPORTED_LIST,
    OID_GEN_HARDWARE_STATUS,
    OID_GEN_MEDIA_SUPPORTED,
    OID_GEN_MEDIA_IN_USE,
    OID_GEN_MAXIMUM_LOOKAHEAD,
    OID_GEN_MAXIMUM_FRAME_SIZE,
    OID_GEN_LINK_SPEED,
    OID_GEN_TRANSMIT_BUFFER_SPACE,
    OID_GEN_RECEIVE_BUFFER_SPACE,
    OID_GEN_TRANSMIT_BLOCK_SIZE,
    OID_GEN_RECEIVE_BLOCK_SIZE,
    OID_GEN_VENDOR_ID,
    OID_GEN_VENDOR_DESCRIPTION,
    OID_GEN_CURRENT_PACKET_FILTER,
    OID_GEN_CURRENT_LOOKAHEAD,
    OID_GEN_DRIVER_VERSION,
    OID_GEN_MAXIMUM_TOTAL_SIZE,
    OID_GEN_PROTOCOL_OPTIONS,
    OID_GEN_MAC_OPTIONS,
    OID_GEN_XMIT_OK,
    OID_GEN_RCV_OK,
    OID_GEN_XMIT_ERROR,
    OID_GEN_RCV_ERROR,
    OID_GEN_RCV_NO_BUFFER,
    OID_802_3_PERMANENT_ADDRESS,
    OID_802_3_CURRENT_ADDRESS,
    OID_802_3_MULTICAST_LIST,
    OID_802_3_MAXIMUM_LIST_SIZE,
    OID_802_3_RCV_ERROR_ALIGNMENT,
    OID_802_3_XMIT_ONE_COLLISION,
    OID_802_3_XMIT_MORE_COLLISIONS
} ;


// driver instance data 
typedef struct 
{
   NDIS_HANDLE NdisMacHandle;
   NDIS_HANDLE NdisWrapperHandle;
   UINT        AdapterCount;
} DRIVER_INSTANCE, *PDRIVER_INSTANCE;
DRIVER_INSTANCE DriverInstance={0};

typedef struct
{
   UCHAR    MacAddress[6];
   UCHAR    MulticastAddress[6];
   ULONG    PacketFilter;
   ULONG    Lookahead;
   ULONG    ProtocolOptions;
} DEVICE_INSTANCE;

BOOLEAN  __cdecl Register_VxD_DDB( int DeviceNum, void *pAdapter );

NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    );

NDIS_STATUS
MiniportInitialize(
    OUT PNDIS_STATUS OpenErrorStatus,
    OUT PUINT SelectedMediumIndex,
    IN PNDIS_MEDIUM MediumArray,
    IN UINT MediumArraySize,
    IN NDIS_HANDLE MiniportAdapterHandle,
    IN NDIS_HANDLE ConfigurationHandle
    );

extern
VOID
MiniportHalt(
    IN NDIS_HANDLE MiniportAdapterContext
    );

NDIS_STATUS
MiniportReset(
    OUT PBOOLEAN AddressingReset,
    IN NDIS_HANDLE MiniportAdapterContext
    );

NDIS_STATUS 
MiniportSend(
        IN NDIS_HANDLE MiniportAdapterContext,
        IN PNDIS_PACKET Packet,
        IN UINT Flags
        );

NDIS_STATUS 
MiniportTransferData(
        OUT PNDIS_PACKET  Packet,
        OUT PUINT  BytesTransferred,
        IN NDIS_HANDLE  MiniportAdapterContext,
        IN NDIS_HANDLE  MiniportReceiveContext,
        IN UINT  ByteOffset,
        IN UINT  BytesToTransfer
        );

NDIS_STATUS
MiniportQueryInformation(
    IN NDIS_HANDLE MiniportAdapterContext,
    IN NDIS_OID Oid,
    IN PVOID InformationBuffer,
    IN ULONG InformationBufferLength,
    OUT PULONG BytesWritten,
    OUT PULONG BytesNeeded
);

extern
NDIS_STATUS
MiniportSetInformation(
    IN NDIS_HANDLE MiniportAdapterContext,
    IN NDIS_OID Oid,
    IN PVOID InformationBuffer,
    IN ULONG InformationBufferLength,
    OUT PULONG BytesRead,
    OUT PULONG BytesNeeded
    );

#pragma NDIS_INIT_FUNCTION(DriverEntry)
NTSTATUS
DriverEntry(
    IN PDRIVER_OBJECT DriverObject,
    IN PUNICODE_STRING RegistryPath
    )

{
   NDIS_STATUS       Status;
   NDIS_MINIPORT_CHARACTERISTICS MiniportChar;
   PDRIVER_INSTANCE  MyDriver = &DriverInstance;
   NDIS_HANDLE       NdisWrapperHandle;

   NdisMInitializeWrapper(&NdisWrapperHandle,
             DriverObject,
             RegistryPath,
             NULL
             );

   MyDriver->NdisWrapperHandle = NdisWrapperHandle;

   MiniportChar.MajorNdisVersion = 3;
   MiniportChar.MinorNdisVersion = 0;
   MiniportChar.CheckForHangHandler      = NULL;
   MiniportChar.DisableInterruptHandler  = NULL;
   MiniportChar.EnableInterruptHandler   = NULL;
   MiniportChar.HandleInterruptHandler   = NULL;             
   MiniportChar.ISRHandler               = NULL;
   MiniportChar.ReconfigureHandler       = NULL;
   MiniportChar.InitializeHandler        = MiniportInitialize;
   MiniportChar.HaltHandler              = MiniportHalt;
   MiniportChar.QueryInformationHandler  = MiniportQueryInformation;
   MiniportChar.ResetHandler             = MiniportReset;
   MiniportChar.SendHandler              = MiniportSend;
   MiniportChar.SetInformationHandler    = MiniportSetInformation;
   MiniportChar.TransferDataHandler      = MiniportTransferData;

   Status = NdisMRegisterMiniport(
              NdisWrapperHandle,
              &MiniportChar,
              sizeof(MiniportChar)
              );

   if (Status == NDIS_STATUS_SUCCESS) 
   {
     return STATUS_SUCCESS;
   }

   return STATUS_UNSUCCESSFUL;

}

#pragma NDIS_PAGEABLE_FUNCTION(MiniportInitialize)
extern
NDIS_STATUS
MiniportInitialize(
    OUT PNDIS_STATUS OpenErrorStatus,
    OUT PUINT SelectedMediumIndex,
    IN PNDIS_MEDIUM MediumArray,
    IN UINT MediumArraySize,
    IN NDIS_HANDLE MiniportAdapterHandle,
    IN NDIS_HANDLE ConfigurationHandle
    )
{
   DEVICE_INSTANCE  *pAdapter;
   UINT              i;
   BOOLEAN           rc;

   for(i = 0; i < MediumArraySize; i++)
   {
      if(MediumArray[i] == NdisMedium802_3)
         break;
   }

   if(i == MediumArraySize)
   {
      return(NDIS_STATUS_UNSUPPORTED_MEDIA);
   }

   *SelectedMediumIndex = i;

   if (NdisAllocateMemory(&pAdapter, sizeof(DEVICE_INSTANCE), 
                          0, MaxPhys) != NDIS_STATUS_SUCCESS)
   {
      return(NDIS_STATUS_RESOURCES);
   }

#ifdef WIN95_BINARY
   rc = Register_VxD_DDB(++DriverInstance.AdapterCount, pAdapter );
#endif

   return NDIS_STATUS_SUCCESS;
}
extern
VOID
MiniportHalt(
    IN NDIS_HANDLE MiniportAdapterContext
    )
{
}

NDIS_STATUS
MiniportReset(
    OUT PBOOLEAN AddressingReset,
    IN NDIS_HANDLE MiniportAdapterContext
    )
{
   return NDIS_STATUS_SUCCESS;
}                                         

NDIS_STATUS
MiniportQueryInformation(
    IN NDIS_HANDLE MiniportAdapterContext,
    IN NDIS_OID Oid,
    IN PVOID InformationBuffer,
    IN ULONG InformationBufferLength,
    OUT PULONG BytesRead,
    OUT PULONG BytesNeeded
)
{
   DEVICE_INSTANCE  *Adapter;
   NDIS_STATUS       Status;
   
   Status = NDIS_STATUS_SUCCESS;
   BytesNeeded = 0;
   Adapter = (DEVICE_INSTANCE *)MiniportAdapterContext;
   switch(Oid)
   {
   case OID_GEN_SUPPORTED_LIST:
      NdisMoveMemory( InformationBuffer, 
                        SupportedOids, 
                        sizeof(SupportedOids) ); 
      *BytesRead=sizeof(SupportedOids); 
      break;

   case OID_GEN_HARDWARE_STATUS:
      *(ULONG *)InformationBuffer = NdisHardwareStatusReady; 
      *BytesRead=sizeof(ULONG); 
      break;

   case OID_GEN_MEDIA_CONNECT_STATUS:
      *(ULONG *)InformationBuffer = NdisMediaStateConnected; 
      *BytesRead = sizeof(ULONG);

   case OID_GEN_MEDIA_SUPPORTED:
   case OID_GEN_MEDIA_IN_USE:
      *(ULONG *)InformationBuffer = NdisMedium802_3; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_MAXIMUM_LOOKAHEAD:
   case OID_GEN_TRANSMIT_BLOCK_SIZE:
   case OID_GEN_RECEIVE_BLOCK_SIZE:
   case OID_GEN_MAXIMUM_TOTAL_SIZE:
   case OID_GEN_TRANSMIT_BUFFER_SPACE:
   case OID_GEN_RECEIVE_BUFFER_SPACE:
   case OID_GEN_MAXIMUM_FRAME_SIZE:
      *(ULONG *)InformationBuffer = MAX_ENET_FRAMESIZE; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_MAXIMUM_SEND_PACKETS:
      *(ULONG *)InformationBuffer = 1; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_LINK_SPEED:
      *(ULONG *)InformationBuffer = 100000; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_VENDOR_ID:
      *(ULONG *)InformationBuffer = 0xFFFFFFFF; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_VENDOR_DESCRIPTION:
      NdisMoveMemory(InformationBuffer, 
                     &VendorName, 
                     sizeof(VendorName) );
      *BytesRead = sizeof(VendorName);

   case OID_GEN_VENDOR_DRIVER_VERSION  :
   case OID_GEN_DRIVER_VERSION:
      *(ULONG *)InformationBuffer = 0x0300; 
      *BytesRead = 2;

   case OID_GEN_CURRENT_PACKET_FILTER:
      *(ULONG *)InformationBuffer = Adapter->PacketFilter; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_CURRENT_LOOKAHEAD      :
      *(ULONG *)InformationBuffer = Adapter->Lookahead; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_802_3_MULTICAST_LIST:
      NdisMoveMemory(InformationBuffer, 
                     Adapter->MulticastAddress, 
                     6 ); 
      *BytesRead = 6; 
      break;

   case OID_802_3_MAXIMUM_LIST_SIZE   :
      *(ULONG *)InformationBuffer = 1;
      *BytesRead = sizeof(ULONG);
      break;

   case OID_GEN_MAC_OPTIONS:
   case OID_GEN_XMIT_OK:
   case OID_GEN_RCV_OK:
   case OID_GEN_XMIT_ERROR:
   case OID_GEN_RCV_ERROR:
   case OID_802_3_RCV_ERROR_ALIGNMENT:
   case OID_802_3_XMIT_ONE_COLLISION:
   case OID_802_3_XMIT_MORE_COLLISIONS:
      *(ULONG *)InformationBuffer = 0; 
      *BytesRead = sizeof(ULONG);
      break;

   case OID_802_3_PERMANENT_ADDRESS:
   case OID_802_3_CURRENT_ADDRESS:
      NdisMoveMemory(InformationBuffer, Adapter->MacAddress, 6 ); 
      *BytesRead = 6; 
      break;

   default:
      Status = NDIS_STATUS_INVALID_OID;
      break;
   }

   return Status;
}
extern
NDIS_STATUS
MiniportSetInformation(
    IN NDIS_HANDLE MiniportAdapterContext,
    IN NDIS_OID Oid,
    IN PVOID InformationBuffer,
    IN ULONG InformationBufferLength,
    OUT PULONG BytesWritten,
    OUT PULONG BytesNeeded
    )
{
   DEVICE_INSTANCE  *Adapter;
   NDIS_STATUS       Status;

   Status = NDIS_STATUS_SUCCESS;
   *BytesNeeded = 0;
   Adapter = (DEVICE_INSTANCE *)MiniportAdapterContext;
   switch(Oid)
   {
   case OID_GEN_CURRENT_PACKET_FILTER:
      Adapter->PacketFilter = *(ULONG *)InformationBuffer; 
      *BytesWritten = sizeof(ULONG);
      break;

   case OID_GEN_CURRENT_LOOKAHEAD:
      Adapter->Lookahead = *(ULONG *)InformationBuffer; 
      *BytesWritten = sizeof(ULONG);
      break;
      
   case OID_GEN_PROTOCOL_OPTIONS:
      // can ignore since we don't have to support a query
      *BytesWritten = sizeof(ULONG);
      break;

   case OID_802_3_MULTICAST_LIST:
      NdisMoveMemory( Adapter->MulticastAddress, 
                      InformationBuffer, 
                      6 );
      *BytesWritten = 6;
      break;

   case OID_802_3_CURRENT_ADDRESS:
      NdisMoveMemory( Adapter->MacAddress, 
                      InformationBuffer, 
                      6 );
      *BytesWritten = 6;
      break;

   default:
      Status = NDIS_STATUS_INVALID_OID;
      break;
   }

   return Status;
}


NDIS_STATUS 
MiniportSend(
        IN NDIS_HANDLE MiniportAdapterContext,
        IN PNDIS_PACKET Packet,
        IN UINT Flags
        )
{
   return NDIS_STATUS_SUCCESS;
}

NDIS_STATUS 
MiniportTransferData(
        OUT PNDIS_PACKET  Packet,
        OUT PUINT  BytesTransferred,
        IN NDIS_HANDLE  MiniportAdapterContext,
        IN NDIS_HANDLE  MiniportReceiveContext,
        IN UINT  ByteOffset,
        IN UINT  BytesToTransfer
        )
{
   return NDIS_STATUS_SUCCESS;
}

